<?php

namespace Drupal\mercury_editor\Controller;

use Drupal\node\Entity\Node;
use Drupal\node\NodeTypeInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Ajax\AjaxHelperTrait;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\node\Controller\NodeViewController;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\mercury_editor\MercuryEditorTempstore;

/**
 * Class EditTrayController.
 *
 * Refreshes the node being edited in the tray.
 */
class MercuryEditorController extends NodeViewController {

  use AjaxHelperTrait;
  use StringTranslationTrait;

  /**
   * The Mercury Editor Edit Tray tempstore service.
   *
   * @var \Drupal\mercury_editor\MercuryEditorTempstore
   */
  protected $tempstore;

  /**
   * Creates a NodeViewController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository.
   * @param \Drupal\mercury_editor\MercuryEditorTempstore $tempstore
   *   The tempstore.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer, AccountInterface $current_user, EntityRepositoryInterface $entity_repository, MercuryEditorTempstore $tempstore) {
    parent::__construct($entity_type_manager, $renderer, $current_user, $entity_repository);
    $this->tempstore = $tempstore;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('current_user'),
      $container->get('entity.repository'),
      $container->get('mercury_editor.tempstore_repository')
    );
  }

  /**
   * Creates a new node of type $node_type and opens the edit tray.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   * @param \Drupal\node\NodeTypeInterface $node_type
   *   The node type.
   */
  public function createNode(Request $request, NodeTypeInterface $node_type) {
    if ($request->getMethod() !== 'GET') {
      $new_node = $this->tempstore->get();
      if ($new_node && $new_node->bundle() !== $node_type->id()) {
        unset($new_node);
      }
    }
    if (empty($new_node)) {
      $type_id = $node_type->id();
      $new_node = Node::create([
        'title' => $this->getTitle($node_type),
        'type' => $type_id,
        'uid' => $this->currentUser->id(),
        'in_preview' => TRUE,
        'preview_view_mode' => 'full',
      ]);
      $this->tempstore->set($new_node);
    }
    $edit_form = \Drupal::entityTypeManager()->getFormObject('node', 'mercury_editor')->setEntity($new_node);
    return \Drupal::formBuilder()->getForm($edit_form);
  }

  /**
   * Preview a node being edited with the mercury editor edit tray.
   */
  public function preview() {
    $node = $this->tempstore->get();
    return $this->view($node);
  }

  /**
   * Access callback for previewing a node being edited with the edit tray.
   */
  public function accessPreview() {
    $node = $this->tempstore->get();
    if ($node) {
      return $node->id() ? $node->access('update', NULL, TRUE) : $node->access('create', NULL, TRUE);
    }
    return AccessResult::forbidden();
  }

  /**
   * {@inheritdoc}
   */
  public function getTitle(NodeTypeInterface $node_type) {
    return $this->t('New :type', [':type' => $node_type->label()]);
  }

}
