<?php

namespace Drupal\mercury_editor;

use Drupal\Core\Url;
use Drupal\node\NodeForm;
use Drupal\node\NodeInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\mercury_editor\Ajax\IFrameAjaxResponseWrapper;
use Drupal\mercury_editor_iframe\Ajax\CloseIframeCommand;
use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\layout_paragraphs\LayoutParagraphsLayoutTempstoreRepository;

/**
 * Node form for mercury editor edit tray.
 */
class MercuryEditorNodeForm extends NodeForm {

  /**
   * The mercury editor tray tempstore repository.
   *
   * @var \Drupal\mercury_editor\MercuryEditorTempstore
   */
  protected $tempstore;

  /**
   * The layout paragraphs tempstore repository.
   *
   * @var \Drupal\layout_paragraphs\LayoutParagraphsLayoutTempstoreRepository
   */
  protected $layoutParagraphsTempstore;

  /**
   * The iFrame Ajax Response Wrapper.
   *
   * @var \Drupal\mercury_editor\Ajax\IFrameAjaxResponseWrapper
   */
  protected $iFrameAjaxResponseWrapper;

  /**
   * The class resolver.
   *
   * @var \Drupal\Core\DependencyInjection\ClassResolverInterface
   */
  protected $classResolver;

  /**
   * Constructs a MercuryEditorNodeForm object.
   *
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The factory for the temp store object.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\mercury_editor\MercuryEditorTempstore $tempstore
   *   The Mercury Edit Edit Tray Tempstore service.
   * @param \Drupal\layout_paragraphs\LayoutParagraphsLayoutTempstoreRepository $layout_paragraphs_tempstore
   *   The Layout Paragraphs Layout Tempstore service.
   * @param \Drupal\mercury_editor\Ajax\IFrameAjaxResponseWrapper $iframe_ajax_response_wrapper
   *   The iFrame Ajax Response Wrapper.
   * @param \Drupal\Core\DependencyInjection\ClassResolverInterface $class_resolver
   *   The class resolver.
   */
  public function __construct(
    EntityRepositoryInterface $entity_repository,
    PrivateTempStoreFactory $temp_store_factory,
    EntityTypeBundleInfoInterface $entity_type_bundle_info = NULL,
    TimeInterface $time = NULL,
    AccountInterface $current_user,
    DateFormatterInterface $date_formatter,
    MercuryEditorTempstore $tempstore,
    LayoutParagraphsLayoutTempstoreRepository $layout_paragraphs_tempstore,
    IFrameAjaxResponseWrapper $iframe_ajax_response_wrapper,
    ClassResolverInterface $class_resolver) {
    parent::__construct($entity_repository, $temp_store_factory, $entity_type_bundle_info, $time, $current_user, $date_formatter);
    $this->tempstore = $tempstore;
    $this->layoutParagraphsTempstore = $layout_paragraphs_tempstore;
    $this->iFrameAjaxResponseWrapper = $iframe_ajax_response_wrapper;
    $this->classResolver = $class_resolver;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.repository'),
      $container->get('tempstore.private'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time'),
      $container->get('current_user'),
      $container->get('date.formatter'),
      $container->get('mercury_editor.tempstore_repository'),
      $container->get('layout_paragraphs.tempstore_repository'),
      $container->get('mercury_editor.iframe_ajax_response_wrapper'),
      $container->get('class_resolver')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    $form = parent::form($form, $form_state);
    if ($this->operation == 'mercury_editor') {
      if ($this->entity->id()) {
        $form['#title'] = $this->t('Edit @type', [
          '@type' => node_get_type_label($this->entity),
        ]);
      }
      else {
        $form['#title'] = $this->t('Create @type', [
          '@type' => node_get_type_label($this->entity),
        ]);
      }
    }
    $form['#attributes']['class'][] = 'me-node-form';
    $form['status_messages'] = [
      '#type' => 'status_messages',
      '#weight' => -100,
    ];

    // Provides a redirect URL so Javascript behaviors can redirect the parent
    // window to the correct destination after the editor is closed.
    $form['redirect_url'] = [
      '#type' => 'hidden',
      '#attributes' => [
        'class' => [
          'me-edit-screen-redirect-url',
        ],
      ],
      '#process' => [[$this, 'processRedirectUrl']],
    ];

    // Sync changes in UI.
    if ($form['title']) {
      $form['title']['widget'][0]['value']['#attributes']['data-sync-changes'] = implode('/', [
        $this->entity->getEntityTypeId(),
        $this->entity->id(),
        'title',
      ]);
    }

    // Temporarily save a reference to the layout paragraphs layout builder in
    // the entity so we can swap the field for the builder when rendering.
    $lp_storage_keys = [];
    foreach (Element::children($form) as $field_name) {
      if (isset($form[$field_name]['widget']['layout_paragraphs_storage_key'])) {
        $lp_storage_keys[$field_name] = $form[$field_name]['widget']['layout_paragraphs_storage_key']['#default_value'];
        // Hide the layout paragraphs field for rendering in the frontend.
        $form[$field_name]['widget']['layout_paragraphs_builder']['#access'] = FALSE;
        $form[$field_name]['widget']['#type'] = 'container';
      }
    }
    if ($lp_storage_keys) {
      $this->entity->lp_storage_keys = $lp_storage_keys;
    }
    if (!$form_state->get('init')) {
      $form_state->set('init', TRUE);
      $this->tempstore->set($this->entity);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {

    $element = parent::actions($form, $form_state);
    if (isset($element['delete'])) {
      $element['delete']['#attributes']['target'] = '_self';
      $query = $element['delete']['#url']->getOption('query') ?? [];
      $element['delete']['#url']->setOption('query', ['_mercury_editor_iframe' => TRUE] + $query);
    }
    $element['submit']['#access'] = TRUE;
    $element['submit']['#ajax'] = [
      'callback' => '::ajaxCallback',
      'progress' => [
        'type' => 'mercuryeditor',
      ],
    ];
    $element['preview']['#access'] = FALSE;
    $element['#attributes']['class'][] = 'me-form-actions';

    return $element;
  }

  /**
   * {@inheritDoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $form_state->setValue('changed', time());
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $form_state->setRebuild(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    // @todo Is this necessary?
    // $this->entity = $this->tempstore->get();
    parent::save($form, $form_state);
    // @todo Is this necessary?
    // $this->tempstore->set($this->entity);
    $this->messenger()->deleteAll();
    $this->messenger()->addStatus($this->t('@type has been updated.', [
      '@type' => $this->entity->type->entity->label(),
    ]));
    foreach (Element::children($form) as $field_name) {
      if (isset($form[$field_name]['widget']['layout_paragraphs_storage_key'])) {
        $key = $form[$field_name]['widget']['layout_paragraphs_storage_key']['#default_value'];
        $layout = $this->layoutParagraphsTempstore->getWithStorageKey($key);
        $layout->setParagraphsReferenceField($this->entity->$field_name);
        $this->layoutParagraphsTempstore->set($layout);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function access(NodeInterface $node, AccountInterface $account) {
    $mercury_content_types = $this->config('mercury_editor.settings')->get('content_types');
    if (!empty($mercury_content_types[$node->bundle()])) {
      return AccessResult::allowed();
    }
    return AccessResult::forbidden();
  }

  /**
   * Ajax callback for rendering the form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    $form['#updated'] = TRUE;
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand('.me-node-form', $form));
    $selector = '[data-me-edit-screen-key="' . $this->entity->uuid() . '"]';
    $node_view = $this->classResolver
      ->getInstanceFromDefinition('\Drupal\node\Controller\NodeViewController')
      ->view($this->entity);
    $this->iFrameAjaxResponseWrapper->addCommand(new ReplaceCommand($selector, $node_view));
    $response->addCommand($this->iFrameAjaxResponseWrapper->getWrapperCommand());
    return $response;
  }

  /**
   * Ajax callback for closing the modal.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function closeModal(array &$form, FormStateInterface $form_state) {
    // Reset the entity by loading it from storage.
    // @todo Use dependency injection.
    $this->entity = \Drupal::entityTypeManager()
      ->getStorage($this->entity->getEntityTypeId())
      ->load($this->entity->id());
    $this->tempstore->set($this->entity);
    $response = new AjaxResponse();
    $response->addCommand(new InvokeCommand('.me-node-form', 'trigger', ['mercury-close-editors']));
    $response->addCommand(new CloseIframeCommand());
    return $response;
  }

  /**
   * Process callback for setting the redirect URL.
   *
   * @param array $element
   *   The form element.
   * @param FormstateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The processed element.
   */
  public function processRedirectUrl(array $element, FormstateInterface $form_state) {
    if ($this->entity->id()) {
      if ($this->entity->isDefaultRevision()) {
        $element['#value'] = $this->entity->toUrl('canonical', ['absolute' => TRUE])->toString();
      }
      else {
        $element['#value'] = $this->entity->toUrl('latest-version', ['absolute' => TRUE])->toString();
      }
    }
    else {
      $element['#value'] = Url::fromRoute('node.add_page', [], ['absolute' => TRUE])->toString();
    }
    return $element;
  }

}
